//  File SerialCamera.pde for camera.
//  25/7/2011 by Piggy
//  Rewritten by Henry Chua to work with e-Gizmo Serial Camera
//  Demo code for using seeeduino or Arduino or gizDuino board to capture jpg format 
//  picture from seeed serial camera and save it into sd card. Push the 
//  button to take the a picture .
 
//  For more details about the product please check http://www.seeedstudio.com/depot/
//  More e-gizmo products can be found http://www.e-gizmo.com

#include <SD.h>


File myFile;

#define PIC_BUF_LEN 64        //data length of each read
/* The buffer size limitation essentially determines the time it takes for the picture
to be saved to a SD card. A VGA resolution color JPEG takes about 30 seconds to save. You
can halve the time by setting the arduino buffer size found in HardwareSerial.cpp 
(e.g. Found in Arduino folder C:\Program Files\arduino-1.0.1\hardware\arduino\cores\arduino) to 128 
(i.e. #define SERIAL_BUFFER_SIZE 128) and change PIC_BUF_LEN 128 accordingly. 
*/


boolean ack;
char  Rx[20];
#define  RST 0x08
#define  SYNC 0x0D
#define  ACK 0x0E
#define  ACKGET 0x0E
#define  INIT 0x01
#define  PSIZE 0x06
#define  CAP 0x05
#define  GETPIC 0x04

char  cmdRST[]={0xAA,0x08,0x01,0x00,0x00,0x00};
char  cmdSYNC[]={0xAA,0x0D,0x00,0x00,0x00,0x00};
char  cmdACK[]={0xAA,0x0E,0x00,0x00,0x00,0x00};
char  cmdACKGET[]={0xAA,0x0E,0x00,0x00,0x00,0x00};
char  cmdINIT[]={0xAA,0x01,0x00,0x07,0x03,0x07};    // Init to 640x480 color JPEG
char  cmdPSIZE[]={0xAA,0x06,0x08,PIC_BUF_LEN,0x00,0x00};
char  cmdCAP[]={0xAA,0x05,0x00,0x00,0x00,0x00};
char  cmdGETPIC[]={0xAA,0x04,0x01,0x00,0x00,0x00};


const int buttonPin = 19;      // the number of the pushbutton pin
unsigned int picTotalLen = 0;  // picture length
int picNameNum = 0;

void setup()
{
  Serial.begin(115200); 
  pinMode(buttonPin,INPUT);    // initialize the pushbutton pin as an input
  digitalWrite(buttonPin,HIGH);
  Serial.println("Initializing SD card....");
  pinMode(10,OUTPUT);          // CS pin of SD Card Shield
  
  if (!SD.begin(10)) {
    Serial.print("initialization failed");
    return;
  }
  Serial.println("initialization done.");
  Serial.setTimeout(10);

  Restart();
}
void loop()
{
  int n = 0;
  while(1){
    Serial.flush();
    Serial.println("Press the button to take a picture");
    while(digitalRead(buttonPin) == HIGH);      //wait for buttonPin status to HIGH
    if(digitalRead(buttonPin) == LOW){
      delay(50);                               //Debounce
      if(digitalRead(buttonPin) == LOW){
        
        Serial.println("COPYING JPG TO CARD, PLS WAIT...");
        Capture();
        delay(500);
        GetData();
       }
      Serial.print("Taking pictures success ,number : ");
      Serial.println(n);
      n ++ ;
      while(digitalRead(buttonPin) == LOW); 
      delay(50);
      }
    }
}

// This routine setup the camera module  
void Restart()
{
  int i;
 
// Sync Arduino/gizDuino board with camera
  while(1){

    delay(500);
    
    sendCmd(cmdSYNC);
    getReply(20,50,SYNC);
  
     if(ack==true) {
         delay(200);
         sendCmd(cmdACK);
         getReply(20,50,ACK);
         break;
       
     }
     else {
       Serial.println("......");
     }
  } 

  // setup the camera operating mode
    sendCmd(cmdRST);
    getReply(20,50,RST);

    sendCmd(cmdINIT);			// Initialize for VGA JPEG color capture
    getReply(20,50,INIT);
    sendCmd(cmdPSIZE);			// set packet size CAP_LEN
    getReply(20,50,PSIZE);

    if(ack==false) Serial.print("Failed to initialize camera.");
      else Serial.println("\nCamera Ready.");

}


boolean  getReply(byte rxsize,byte dly,char inst){

    int i=0;
    ack=false;
   
   delay(dly);
   
    Serial.readBytes(Rx,rxsize);
    
    if(Rx[1]==0x0E){
        if((Rx[2]==inst)|(Rx[2]==0)){
          ack=true;
          return(true);  
        }
        Serial.print("~E");		// ~E appearing means error in comm occured
    } else return(false);

}

void Capture()
{
  delay(1000);            // This delay will ensure camera is ready to take pic
  sendCmd(cmdRST);
  getReply(20,50,RST);
  sendCmd(cmdCAP);
  while(getReply(20,50,CAP)==false);
}


boolean GetData()
{

  unsigned int picSize;
  unsigned  char idlow=0;
  unsigned  char  idhigh=0;
  int  packetsize;
  
  // Initiate JPG Picture file transfer

  
  sendCmd(cmdGETPIC);
  while(getReply(20,150,GETPIC)==false);

  if(Rx[7] != 0x0A) { Serial.println("Error encountered with camera."); return(false); }
  
  // Fetch Picture Size

  picSize= (Rx[9]&0xFF)+(Rx[10]&0xFF)*256;

  // SET ACK function as normal ACK
  cmdACKGET[4] = 0;
  cmdACKGET[5] = 0;
  


  // Generate Picture File name
  char picName[] = "pic00.jpg";
  picName[3] = picNameNum/10 + '0';
  picName[4] = picNameNum%10 + '0';
  
  myFile = SD.open(picName, FILE_WRITE);
  if(!myFile){
    Serial.println("myFile open fail...");
  }
  else{
        while(1){   
      // Get as packet
      sendCmd(cmdACKGET);
      getReply(4,52,00);

      // Data Size in this packet
      packetsize=Rx[2]&0xFF;
      
      readCamSaveToFile(myFile, packetsize);    // save to SD
      
      // If packet size is less than expected, EOF is reached
      if(packetsize<(PIC_BUF_LEN-6)) break;
      
      // upadate ACK packet ID
      idlow++;
      if(idlow==0) idhigh++;

      cmdACKGET[4]=idlow;
      cmdACKGET[5]=idhigh;
      

    }
    // EOF, tell camera to stop transfer mode
    cmdACKGET[4] = 0xF0;
    cmdACKGET[5] = 0xF0;
    sendCmd(cmdACKGET);
    
    getReply(20,500,00);
     
  }
  myFile.close();
  picNameNum ++;
  
  Serial.print("\nSize: ");
  Serial.println(picSize);
  return(true);
}


// Save JPG packet to SD
void readCamSaveToFile(File &myFile,int toBeReadLen)
{
  char readLen = 0;
  
  while(readLen < toBeReadLen){		
      while(Serial.available()==0) delay(3);
      myFile.write(Serial.read());
      readLen++;
  }

}

void sendCmd(char cmd[] )
{
  while(Serial.available()>0)  char t=Serial.read();   // Flush RX buffer
  for(char i = 0; i < 6; i++)Serial.print(cmd[i]);  // send command
}


